
#define F_CPU 8000000UL
#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/sfr_defs.h>


#define SET_BIT(R, bit) R |= (1 << bit)
#define CLR_BIT(R, bit) R &= ~(1 << bit)

int led_power=0;
int input_lux=0;
int mode=0;
char lcd[20]="";

#define FIRST_ADC_INPUT 0
#define LAST_ADC_INPUT 1
// Voltage Reference: AREF pin
#define ADC_VREF_TYPE ((0<<REFS1) | (0<<REFS0) | (0<<ADLAR))


unsigned char input_index=0;
unsigned int adc_data[2];

ISR(INT0_vect)
{
	//toggle between auto and manual
	if (mode==0)
	{
		mode=1;
	}
	else
	{
		mode=0;
		led_power=100;
	}
}

ISR(ADC_vect)
{
	
	//AD conversion result
	adc_data[input_index]=ADCW;
	// Select the next ADC input
	if (++input_index > 1)
	input_index=0;
	ADMUX=(FIRST_ADC_INPUT | ADC_VREF_TYPE)+input_index;
	// Delay needed for the stabilization of the ADC input voltage
	_delay_us(10);
	// Start the AD conversion
	ADCSRA|=(1<<ADSC);
}

ISR(TIMER1_OVF_vect)//3s timer
{
	TCNT1=42099;
	
	if (mode==0)//if mode is auto
	{
		if (input_lux>1000)//max range
		{
			if (led_power>=20)
			{
				led_power=led_power-20;
			}
		}
		
		if (input_lux<800)//min range
		{
			if (led_power<=80)
			{
				led_power=led_power+20;
			}
		}
	}
	
}

int uart_putchar(char c, FILE *stream)
{

	if (c == '\n')
	uart_putchar('\r', stream);
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = c;
	return 0;
}

int main(void)
{
	//division factor: 1
	CLKPR=(1<<CLKPCE);
	CLKPR=(0<<CLKPCE) | (0<<CLKPS3) | (0<<CLKPS2) | (0<<CLKPS1) | (0<<CLKPS0);
	
	/* Replace with your application code */
	DDRB=0b11111111;//output port for lcd
	PORTB=0;
	
	DDRC=0b11111100;//output port for led
	
	DDRD=0b11111010;

	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 7.813 kHz
	// Mode: Normal top=0xFFFF
	// OC1A output: Disconnected
	// OC1B output: Disconnected
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer Period: 2.9999 s
	// Timer1 Overflow Interrupt: On
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TCCR1A=(0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0) | (0<<WGM11) | (0<<WGM10);
	TCCR1B=(0<<ICNC1) | (0<<ICES1) | (0<<WGM13) | (0<<WGM12) | (1<<CS12) | (0<<CS11) | (1<<CS10);
	TCNT1H=0xA4;
	TCNT1L=0x73;
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;
	
	// Timer/Counter 1 Interrupt(s) initialization
	TIMSK1=(0<<ICIE1) | (0<<OCIE1B) | (0<<OCIE1A) | (1<<TOIE1);
	
	
	// External Interrupt(s) initialization
	// INT0: On
	// INT0 Mode: Rising Edge
	EICRA=(0<<ISC11) | (0<<ISC10) | (1<<ISC01) | (1<<ISC00);
	EIMSK=(0<<INT1) | (1<<INT0);
	EIFR=(0<<INTF1) | (1<<INTF0);
	
	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: Off
	// USART Transmitter: On
	// USART0 Mode: Asynchronous
	// USART Baud Rate: 9600 (Double Speed Mode)
	UCSR0A=(0<<RXC0) | (0<<TXC0) | (0<<UDRE0) | (0<<FE0) | (0<<DOR0) | (0<<UPE0) | (1<<U2X0) | (0<<MPCM0);
	UCSR0B=(0<<RXCIE0) | (0<<TXCIE0) | (0<<UDRIE0) | (0<<RXEN0) | (1<<TXEN0) | (0<<UCSZ02) | (0<<RXB80) | (0<<TXB80);
	UCSR0C=(0<<UMSEL01) | (0<<UMSEL00) | (0<<UPM01) | (0<<UPM00) | (0<<USBS0) | (1<<UCSZ01) | (1<<UCSZ00) | (0<<UCPOL0);
	UBRR0H=0;
	UBRR0L=103;
	
	FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,
	_FDEV_SETUP_WRITE);
	stdout = &mystdout;
	
	// ADC initialization
	// ADC Clock frequency: 1000.000 kHz
	// ADC Voltage Reference: AREF pin
	// ADC Auto Trigger Source: Free Running
	// Digital input buffers on ADC0: On, ADC1: On, ADC2: On, ADC3: On
	// ADC4: On, ADC5: On
	DIDR0=(0<<ADC5D) | (0<<ADC4D) | (0<<ADC3D) | (0<<ADC2D) | (0<<ADC1D) | (0<<ADC0D);
	ADMUX=FIRST_ADC_INPUT | ADC_VREF_TYPE;
	ADCSRA=(1<<ADEN) | (1<<ADSC) | (1<<ADATE) | (0<<ADIF) | (1<<ADIE) | (0<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
	ADCSRB=(0<<ADTS2) | (0<<ADTS1) | (0<<ADTS0);
	
	lcd_init(LCD_DISP_ON);

	
	//enable global intrupts
	sei();
	
	lcd_clrscr();
	_delay_ms(100);
	while (1)
	{
		input_lux=(-6.134*adc_data[0])+1827;//calculate lux
		lcd_gotoxy(0,0);
		sprintf(lcd,"Light:%u LUX ",input_lux);
		lcd_puts(lcd);
		lcd_gotoxy(0,1);
		sprintf(lcd,"LED:%u%% ",led_power);
		lcd_puts(lcd);
		
		printf("Light:%u LUX     ",input_lux);
		
		lcd_gotoxy(8,1);
		if (mode==0)
		{
			lcd_puts("M:Auto  ");
			printf("Mode: Auto    ");
		}
		else
		{
			led_power=adc_data[1]/10;//convert 0-1024 to 0-100
			if (led_power>100)
			{
				led_power=100;
			}
			
			lcd_puts("M:Manual");
			printf("Mode: Manual    ");
			
		}
		
		if(led_power<=10)
		{
			
			CLR_BIT(PORTC, PORTC2);
			CLR_BIT(PORTC, PORTC3);
			CLR_BIT(PORTC, PORTC4);
			CLR_BIT(PORTC, PORTC5);
			CLR_BIT(PORTC, PORTC6);
		}
		else if(led_power>10 && led_power<=20)
		{
			SET_BIT(PORTC, PORTC2);
			CLR_BIT(PORTC, PORTC3);
			CLR_BIT(PORTC, PORTC4);
			CLR_BIT(PORTC, PORTC5);
			CLR_BIT(PORTC, PORTC6);
		}
		else if(led_power>20 && led_power<=40)
		{
			SET_BIT(PORTC, PORTC2);
			SET_BIT(PORTC, PORTC3);
			CLR_BIT(PORTC, PORTC4);
			CLR_BIT(PORTC, PORTC5);
			CLR_BIT(PORTC, PORTC6);
		}
		else if(led_power>40 && led_power<=60)
		{
			SET_BIT(PORTC, PORTC2);
			SET_BIT(PORTC, PORTC3);
			SET_BIT(PORTC, PORTC4);
			CLR_BIT(PORTC, PORTC5);
			CLR_BIT(PORTC, PORTC6);
		}
		else if(led_power>60 && led_power<=80)
		{
			
			SET_BIT(PORTC, PORTC2);
			SET_BIT(PORTC, PORTC3);
			SET_BIT(PORTC, PORTC4);
			SET_BIT(PORTC, PORTC5);
			CLR_BIT(PORTC, PORTC6);
		}
		else if(led_power>80)
		{
			
			SET_BIT(PORTC, PORTC2);
			SET_BIT(PORTC, PORTC3);
			SET_BIT(PORTC, PORTC4);
			SET_BIT(PORTC, PORTC5);
			SET_BIT(PORTC, PORTC6);
		}
		
		//send to serial
		printf("    LED Power:%u%% \n",led_power);
		_delay_ms(500);
		
	}
}

